import {
  PropType,
  Teleport,
  computed,
  defineComponent,
  onMounted,
  reactive,
  watch
} from 'vue';
import {
  IApiUserPaymentOrderPage,
  api_userPaymentOrderPage,
  api_userPaymentOrderStudentRefundOrderPage,
  type_OrderStatus
} from '../api';
import Item from './item';
import MFullRefresh from '@/components/m-full-refresh';
import { Button, DatePicker, List, Popup, showToast } from 'vant';
import MEmpty from '@/components/m-empty';
import ItemSkelete from './item-skelete';
// import icon_arrow from '../image/icon_arrow.svg';
import styles from './list.module.less';
// import { usePageVisibility } from '@vant/use';
import iconArrowDown from '../image/icon-arrow.png';
import iconArrowDownActive from '../image/icon-arrow-active.png';
import { listenerMessage } from '@/helpers/native-message';
import TheTimeRange from '@/components/the-time-range';

export default defineComponent({
  name: 'collection-record-list',
  props: {
    orderStatus: {
      type: String,
      default: ''
    },
    refundIndex: {
      type: Number,
      default: 0
    }
  },
  emits: ['refund'],
  setup(props, { emit }) {
    const time = new Date();
    const data = reactive({
      page: 1,
      // dateTime: [
      //   time.getFullYear().toString(),
      //   (time.getMonth() + 1).toString().padStart(2, '0')
      // ],
      startTime: null as any,
      endTime: null as any,
      minDate: new Date(time.getFullYear(), 0, 1),
      maxDate: new Date(time.getFullYear() + 10, 5, 1),

      list: [],
      skelete: true,
      refreshing: false,
      finished: false,
      loading: false,

      dataOpen: false
    });
    const statusTypes: { [_: string]: string } = {
      WAIT_PAY: '待支付',
      PAYING: '待支付',
      PAID: '支付成功',
      CLOSED: '已关闭',
      REFUNDED: '退款成功'
    };
    const statusColors: { [_: string]: string } = {
      WAIT_PAY: 'rgba(64, 196, 203, 1)',
      PAYING: 'rgba(64, 196, 203, 1)',
      PAID: 'rgba(28, 172, 241, 1)',
      CLOSED: 'rgba(170, 170, 170, 1)',
      ING: 'rgba(153, 153, 153, 1)',
      REJECT: 'rgba(255, 90, 86, 1)',
      PASS: 'rgba(255, 162, 68, 1)',
      REFUNDED: 'rgba(255, 162, 68, 1)'
    };
    const refundTypes: { [_: string]: string } = {
      ING: '退款申请中',
      REJECT: '退款失败',
      PASS: '退款成功',
      CLOSED: '退款关闭'
    };
    const getList = async () => {
      data.loading = true;
      let res = {} as any;
      if (props.orderStatus === 'REFUNDED') {
        const params = {
          page: data.page,
          rows: 10,
          startTime: null as any,
          endTime: null as any
        };
        if (data.startTime && data.endTime) {
          params.startTime = data.startTime + ' 00:00:00';
          params.endTime = data.endTime + ' 23:59:59';
        }
        res = await api_userPaymentOrderStudentRefundOrderPage(params);
        if (res?.code === 200 && Array.isArray(res?.data?.rows)) {
          data.list = res.data.rows.map((item: any) => {
            return {
              ...item,
              paymentCashAmount: item.paymentCashAmount?.toFixed(2) || '0.00',
              createTime: item.refundTime,
              statusName: refundTypes[item.refundAudit],
              statusColor: statusColors[item.refundAudit],
              isRefund: true // 是退款列表
            };
          });
          data.finished = !res.data.next;
          data.page = data.page + 1;
        }
      } else {
        try {
          const params = {
            page: data.page,
            rows: 10,
            orderStatus: props.orderStatus as any,
            startTime: null as any,
            endTime: null as any
          };
          if (data.startTime && data.endTime) {
            params.startTime = data.startTime + ' 00:00:00';
            params.endTime = data.endTime + ' 23:59:59';
          }
          res = await api_userPaymentOrderPage(params);
        } catch (error) {}

        if (res?.code === 200 && Array.isArray(res?.data?.rows)) {
          data.list = res.data.rows.map((item: any) => {
            return {
              ...item,
              paymentCashAmount: item.paymentCashAmount?.toFixed(2) || '0.00',
              createTime: item.payTime,
              statusName: statusTypes[item.status],
              statusColor: statusColors[item.status]
            };
          });
          data.finished = !res.data.next;
          data.page = data.page + 1;
        }
      }

      setTimeout(() => {
        data.refreshing = false;
      }, 500);
      data.loading = false;
      data.skelete = false;
    };

    watch(
      () => props.refundIndex,
      () => handlStart()
    );
    onMounted(() => {
      getList();
      listenerMessage('webViewOnResume', () => handlStart());
    });

    const handlStart = () => {
      data.finished = false;
      data.page = 1;
      getList();
    };

    // const timeName = computed(() => {
    //   return `${data.dateTime[0]}年${data.dateTime[1]}月`;
    // });
    return () => (
      <>
        <MFullRefresh
          modelValue={data.refreshing}
          onRefresh={() => {
            data.refreshing = true;
            console.log('下拉');
            handlStart();
          }}>
          <div class={styles.prodSection}>
            <div class={styles.title}>选择时间</div>
            <div
              class={[styles.times, data.dataOpen ? styles.active : '']}
              onClick={() => {
                data.dataOpen = true;
              }}>
              <span>筛选</span>
              <img
                class={styles.iconArrow}
                src={data.dataOpen ? iconArrowDownActive : iconArrowDown}
              />
            </div>
          </div>
          {data.skelete && new Array(5).fill(1).map(i => <ItemSkelete />)}

          {data.list.length !== 0 && (
            <List
              loading={data.loading}
              finished={data.finished}
              finishedText=" ">
              {data.list.map(item => {
                return (
                  <Item
                    item={item}
                    onClose={() => handlStart()}
                    onRefund={() => emit('refund')}
                  />
                );
              })}
            </List>
          )}

          {!data.skelete && data.list.length === 0 && (
            <MEmpty description="暂无记录" />
          )}
        </MFullRefresh>
        {/* <Popup
          teleport="body"
          v-model:show={data.dataOpen}
          position="bottom"
          round>
          <DatePicker
            v-model={data.dateTime}
            columnsType={['year', 'month']}
            minDate={data.minDate}
            maxDate={data.maxDate}
            onConfirm={() => {
              data.dataOpen = false;
              console.log(data.dateTime);
              handlStart();
            }}
            onCancel={() => (data.dataOpen = false)}
          />
        </Popup> */}
        <Teleport to={'body'}>
          <TheTimeRange
            v-model:show={data.dataOpen}
            onConfirm={(val: any) => {
              console.log(val, 'val');

              data.dataOpen = false;
              data.startTime = val.startTime;
              data.endTime = val.endTime;
              handlStart();
            }}
          />
        </Teleport>
      </>
    );
  }
});
