import { computed, defineComponent, ref } from 'vue';
import { RouterView, useRoute } from 'vue-router';
import useAuthCode from '@/hooks/useAuthCode';
import { browser, getHttpOrigin } from '@/helpers/utils';
import qs from 'query-string';

export default defineComponent({
  name: 'wx-auth-layout',
  setup() {
    const authCode = useAuthCode();
    const route = useRoute();
    const loading = ref(false);
    const error = ref(false); // 如果授权失败了也可以正常访问页面
    const openId = ref('');
    const isNeedView = computed(() => {
      return openId.value || error.value || !browser().weixin ? true : false;
    });

    const setAuth = async () => {
      if (loading.value) {
        return;
      }

      loading.value = true;
      try {
        openId.value = await authCode.getOpenId(
          getHttpOrigin() +
            window.location.pathname +
            '#' +
            route.path +
            '?' +
            qs.stringify({
              ...route.query
            })
        );
      } catch (e: any) {
        //
        console.log(e, 'error');
        error.value = true;
      }
      loading.value = false;
    };

    setAuth();

    return () => (isNeedView.value ? <RouterView></RouterView> : null);
  }
});
